#################################################################################
#                                                                               #
#  65C02 Simulator for PI  v1.0                             Feb 23, 2018        #
#                                                                               #
#  Written by Daryl Ritor with credit to David Welch's awesome tutorials        #
#                                                                               #
#################################################################################
#
# Start of program ... 
#
.global _start
_start:
   b init			@ jump around the space opned up below
#
# Create space for the 65C02 memory system. This file gets loaded at 0x8000 by
# the PI firmware, so I add 0x8000 more space (minus 4 for the 'b init' instruction)
# for 64K.  This simulation configures it's memory  like this:
#    0x0000 - 0x7FFF - RAM
#    0x8000 - 0x80FF - IO   
#    0x8100 - 0xFFFF - ROM
#
# This configuration is easy to change, as long as the IO page is the boarder between
# RAM and ROM.  It's also easy to make make the ROM into RAM.  Any other configuration
# will come with a speed penalty.
#
.space 0x7FFC			@ add space for 65C02 memory

init:
# set stack pointer above program code (do I need other stacks set up too?)
   mov sp, #0x20000

########################################################################
# configure PI hardware - pg references are to the BCM Hardware Manual #
########################################################################
# Invalidate the caches

   mov r2, #0			@ zero
   mcr p15, 0, r2, c7, c7, 0 	@ invalidate both caches (pg 3-70)
   mcr p15, 0, r2, c8, c7, 0 	@ Unified TLB, Invalidate TLB unlocked entries (pg 3-86)
   mcr p15, 0, r2, c7,c10, 4 	@ Data Syncronization Barrier - flushes the write buffers (pg 3-83)

# The MMU can support up to 16 Domains, I only use one but am setting them all the same
# Each Domain gets 2 bits within this word, 11b = Manager Mode and Don't fault on permissions (pg 3-63)
   mvn r2, #0			@ 0xFFFFFFFF
   mcr p15,0,r2,c3,c0,0         @ set all MMU domains to 11b

#  This is how I am using the PI's RAM
#
#  0x00000000 - 0x0000FFFF - 65C02 System memory RAM and Simulated ROM
#  0x00100000 - 0x0017FFFF - ARM program space
#  0x00180000 - 0x001BFFFF - MMU translation table (virtual to physical memory map)
#  0x001C0000 - 0x001FFFFF - ARM System Stack (won't get used very much)
#  0x00200000 - 0x002FFFFF - Bootloader (for testing - not used in Simulation)

#  Set up the MMU translation table with 1MB blocks. This is our configuration:
#
#  0x00000000 - 0x000FFFFF - Cachable & bufferable
#  0x00100000 - 0x001FFFFF - Cachable & bufferable
#  0x00200000 - 0x002FFFFF - Cachable & bufferable
#  0x20000000 - 0x200FFFFF - NonCachable & nonbufferable - system IO space
#  0x20200000 - 0x202FFFFF - NonCachable & nonbufferable - system IO space

   mov r3, #0x00180000		@ MMU Table Base address
   mov r4, #0x0000000E		@ 1st entry, cache, buf, 1MB, 0x000
   str r4, [r3], #4		@ write entry 0x0000000E -> 0x00180000
   orr r4, r4, #0x00100000	@ 2nd entry, cache, buf, 1MB, 0x001
   str r4, [r3], #4 		@ write entry 0x0010000E -> 0x00180004
   add r4, r4, #0x00100000	@ 3rd entry, cache, buf, 1MB, 0x002
   str r4, [r3] 		@ write entry0x0020000E -> 0x00180008
   mov r3, #0x00180000		@ MMU Table Base address
   orr r3, r3, #0x800		@ offset to 0x200 data point - 0x00108800
   mov r4, #0x20000000		@ address is 0x200
   orr r4, r4, #0x02		@ IO entry, no cache, no buffer, 1MB, 0x200
   str r4, [r3] 		@ write entry 0x20000002 -> 0x00180800
   orr r3, r3, #0x008		@ Offset to 0x202 data point
   orr r4, r4, #0x200000	@ IO entry, no cache, no buffer, 1MB, 0x202
   str r4, [r3] 		@ write entry 0x20200002 -> 0x00180808

   mov r0, #0x00180000		@ MMU Table Base address
   mcr p15, 0, r0, c2, c0, 0 	@ Set tlb base for process specific apps(pg 3-57)
   mcr p15, 0, r0, c2, c0, 1 	@ Set tlb base for OS and IO addresses (pg 3-59)

#
# Configure and Enable MMU and Caches
#
   mrc p15, 0, r2, c1, c0, 0	@ read control register (pg 3-47)
   bic r2, r2, #0x02		@ A - no unaligned access fault (0)
   orr r2, r2, #1 << 22         @ U - unaligned access model
   orr r2, r2, #0x1000 		@ I - instruction cache enable (pg 3-47)
   orr r2, r2, #0x800		@ Z - enable branch prediction (pg 3-46)
   orr r2, r2, #0x04 		@ C - data cache enable (pg 3-47)
   orr r2, r2, #0x01		@ M - enable MMU
   mcr p15, 0, r2, c1, c0, 0    @ write control register (pg 3-47)
#
# Set up PI IO Peripherals
#
   .set GPFSEL1,	 0x20200004	@IO
   .set GPPUD,		 0x20200094	@UART
   .set GPPUDCLK0,	 0x20200098
   .set AUX_ENABLES,	 0x20215004
   .set AUX_MU_IO_REG,	 0x20215040
   .set AUX_MU_IER_REG,	 0x20215044
   .set AUX_MU_IIR_REG,	 0x20215048
   .set AUX_MU_LCR_REG,	 0x2021504C
   .set AUX_MU_MCR_REG,	 0x20215050
   .set AUX_MU_LSR_REG,	 0x20215054
   .set AUX_MU_CNTL_REG, 0x20215060
   .set AUX_MU_BAUD_REG, 0x20215068
   .set GPSET0,  	 0x2020001C	@LED
   .set GPCLR0,		 0x20200028	@LED
   .set SYSTIMERCLO,	 0x20003004     @Timer

LEDinit:
   ldr r0, =GPFSEL1
   ldr r1, [r0]
   bic r1, r1, #0x1C0000		@ AND with inverted second term
   orr r1, r1, #0x40000
   str r1, [r0] 

UARTinit:
   ldr r0, =AUX_ENABLES		@ 115200,N,8,1
   mov r1, #1
   str r1, [r0]
   ldr r0, =AUX_MU_IER_REG
   mov r1, #0
   str r1, [r0]
   ldr r0, =AUX_MU_CNTL_REG
   str r1, [r0]
   ldr r0, =AUX_MU_LCR_REG
   mov r1, #3
   str r1, [r0]
   ldr r0, =AUX_MU_MCR_REG
   mov r1, #0
   str r1, [r0]
   ldr r0, =AUX_MU_IER_REG
   str r1, [r0]
   ldr r0, =AUX_MU_IIR_REG
   mov r1, #0xC6
   str r1, [r0]
   ldr r0, =AUX_MU_BAUD_REG
   ldr r1, =270			@ 250,000,000 / (8 *115,200) - 1			
   str r1, [r0]
				
   ldr r0, =GPFSEL1
   ldr r1, [r0]
   bic r1, r1, #0x3F000		@ AND with inverted second term
   orr r1, r1, #0x12000
   str r1, [r0] 

   ldr r0, =GPPUD
   mov r1, #0
   str r1, [r0]
   mov r0, #352			@ delay 0x160
.l1:   
   sub r0, r0, #1
   uxtb r0, r0
   cmp r0, #0
   bne .l1

   ldr r0, =GPPUDCLK0
   mov r1, #0xC000		
   str r1, [r0]
   mov r0, #352			@ delay 0x160
.l2:   
   sub r0, r0, #1
   uxtb r0, r0
   cmp r0, #0
   bne .l2

   ldr r0, =GPPUDCLK0
   mov r1, #0
   str r1, [r0]
   ldr r0, =AUX_MU_CNTL_REG
   mov r1, #3
   str r1, [r0]
   b  CPUinit

######################################### 
# Routines used by Simulator's IO calls #
#########################################
#
# Read IO    			R0 = address, R1 = data returned
#
.global uart_txrdy		
uart_txrdy:		
   ldr r1, =AUX_MU_LSR_REG
   ldr r1, [r1]
   ands r1, r1, #0x20   	@ mask TX ready bit
   movne r1, #0xFF
   bx lr			@ R1=0 if not ready, R1=0xFF if ready

.global uart_getc		
uart_getc:			
   ldr r1, =AUX_MU_IO_REG
   ldr r1, [r1]
   bx lr			@ return with char in R1

.global uart_scan
uart_scan:		
   ldr r1, =AUX_MU_LSR_REG
   ldr r1, [r1]
   ands r1, r1, #0x01   	@ test for receive data
   movne r1, #0xFF
   bx lr			@ R1=0 if no data, R1=0xFF if data

.global ticks
ticks:			
   ldr r1, =SYSTIMERCLO
   ldr r1, [r1] 
   bx lr

#
# write IO			r0 = address, r1 = data to send
#
.global uart_putc	
uart_putc:
   ldr r0, =AUX_MU_LSR_REG
   ldr r0, [r0]
   ands r0, r0, #0x20   	@ mask TX ready bit
   beq uart_putc
   ldr r0, =AUX_MU_IO_REG
   str r1, [r0]			@ store data in R1 to UART
   bx lr

.global LED_off
LED_off:
   ldr r0, =GPCLR0
   ldr r1, =#0x10000
   str r1, [r0] 
   bx lr

.global LED_on
LED_on:
   ldr r0, =GPSET0
   ldr r1, =#0x10000
   str r1, [r0] 
   bx lr

#
# done
#
