#################################################################################
#                                                                               #
#  65C02 Simulator for PI  v1.0                             Feb 23, 2018        #
#                                                                               #
#  Written by Daryl Ritor with credit to David Welch's awesome tutorials        #
#                                                                               #
#################################################################################
#
# Memory access and IO support
#
.global mem_init
mem_init:			@ fill 64k memory image 
   ldr r1, =mem			@ pointer to Memory contents (see mem: statements below)
   mov r2, #0			@ pointer to RAM -> 0x00000000
   ldr r3, =0x10000		@ end of copy value 0xFFFF + 1
memlp:
   ldrb r0, [r1], #1		@ get from mem and inc pointer
   strb r0, [r2], #1   		@ store to RAM and inc pointer
   cmp r2, r3			@ does target = end vaue?
   bne memlp			@ no, repeat
   bx lr			@ return
#
.ltorg	@ literal storage	@ for holding litteral pool values
#

############################
# define IO READ functions #
############################
#
# This is using a series of compare and branch statements, as more Io support
# is added, I'll need to do this more efficiently.
#
.global readio
readio:				@addr=r0, data=r1
	andeq r1, r0, #0xFF
	cmp r1, #0x00
	beq uart_getc		@ 00: uart_getc
        cmp r1, #0x01	
        beq uart_scan		@ 01: uart_scan
        cmp r1, #0x10	
        beq uart_txrdy		@ 10: uart_txrdy
        cmp r1, #0x30	
        beq ticks		@ 30: tick counter
        mov r1, #0		@ else return 0
	bx lr

#############################
# define IO WRITE functions #
#############################
#
# This is using a series of compare and branch statements, as more Io support
# is added, I'll need to do this more efficiently.
#
.global writeio
writeio:		@addr=R0, data=R1
	andeq r0, r0, #0xFF
	cmp R0, #0x10
	beq uart_putc		@ 00: uart_putc
	cmp R0, #0x20
	beq LED_off		@ 20: LED_off
	cmp R0, #0x21
	beq LED_on		@ 21: LED_on
	bx lr

#################
# define memory #
#################			@ copy of our memory at startup
mem: .space 0x8000			@ RAM 0x0000 - 0x7FFF, zeroed
     .incbin "sbcos.rom", 0, 32768	@ SBC2OS in ROM from 0x8000 - 0xFFFF (first page is masked by IO)

@mem:  .incbin "testc.bin", 0, 65536	@ Klaus's test suites take the whole 64k
